/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */
package de.justjanne.libquassel.protocol.serializers.signalproxy

import de.justjanne.libquassel.protocol.models.SignalProxyMessage
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.syncables.common.Network
import de.justjanne.libquassel.protocol.syncables.state.NetworkState
import de.justjanne.libquassel.protocol.testutil.byteBufferOf
import de.justjanne.libquassel.protocol.testutil.signalProxySerializerTest
import org.junit.jupiter.api.Tag
import org.junit.jupiter.api.Test

@Tag("SignalProxySerializerTest")
class InitDataSerializerTest {
  @Test
  fun testEmptyMap() = signalProxySerializerTest(
    SignalProxyMessage.InitData(
      className = "",
      objectName = "",
      initData = emptyMap()
    ),
    byteBufferOf(
      // 4 elements
      0x00u, 0x00u, 0x00u, 0x01u,
      // int
      0x00u, 0x00u, 0x00u, 0x02u,
      0x00u,
      // Rpc
      0x00u, 0x00u, 0x00u, 0x04u,
    ),
    featureSets = emptyList(),
    serializeFeatureSet = null
  )

  @Test
  fun testSimple() = signalProxySerializerTest(
    SignalProxyMessage.InitData(
      className = "",
      objectName = "",
      initData = emptyMap()
    ),
    byteBufferOf(
      0x00u, 0x00u, 0x00u, 0x03u, 0x00u, 0x00u, 0x00u, 0x02u, 0x00u, 0x00u, 0x00u, 0x00u, 0x04u, 0x00u, 0x00u, 0x00u,
      0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
    )
  )

  @Test
  fun testRealistic() {
    signalProxySerializerTest(
      SignalProxyMessage.InitData(
        className = "Network",
        objectName = "4",
        initData = Network(
          state = NetworkState(
            networkId = NetworkId(4)
          )
        ).toVariantMap()
      ),
      byteBufferOf(
        0x00u, 0x00u, 0x00u, 0x45u, 0x00u, 0x00u, 0x00u, 0x02u, 0x00u, 0x00u, 0x00u, 0x00u, 0x04u, 0x00u, 0x00u, 0x00u,
        0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x4Eu, 0x65u, 0x74u, 0x77u, 0x6Fu, 0x72u, 0x6Bu, 0x00u, 0x00u, 0x00u,
        0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x01u, 0x34u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu,
        0x6Eu, 0x65u, 0x74u, 0x77u, 0x6Fu, 0x72u, 0x6Bu, 0x4Eu, 0x61u, 0x6Du, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u,
        0x00u, 0x00u, 0x00u, 0x22u, 0x00u, 0x3Cu, 0x00u, 0x6Eu, 0x00u, 0x6Fu, 0x00u, 0x74u, 0x00u, 0x20u, 0x00u, 0x69u,
        0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x61u, 0x00u, 0x6Cu, 0x00u, 0x69u, 0x00u, 0x7Au,
        0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x3Eu, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Du, 0x63u,
        0x75u, 0x72u, 0x72u, 0x65u, 0x6Eu, 0x74u, 0x53u, 0x65u, 0x72u, 0x76u, 0x65u, 0x72u, 0x00u, 0x00u, 0x00u, 0x0Au,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x06u, 0x6Du, 0x79u,
        0x4Eu, 0x69u, 0x63u, 0x6Bu, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x6Cu, 0x61u, 0x74u, 0x65u, 0x6Eu, 0x63u, 0x79u, 0x00u, 0x00u, 0x00u,
        0x02u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Eu, 0x63u,
        0x6Fu, 0x64u, 0x65u, 0x63u, 0x46u, 0x6Fu, 0x72u, 0x53u, 0x65u, 0x72u, 0x76u, 0x65u, 0x72u, 0x00u, 0x00u, 0x00u,
        0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x05u, 0x55u, 0x54u, 0x46u, 0x5Fu, 0x38u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u,
        0x00u, 0x00u, 0x00u, 0x10u, 0x63u, 0x6Fu, 0x64u, 0x65u, 0x63u, 0x46u, 0x6Fu, 0x72u, 0x45u, 0x6Eu, 0x63u, 0x6Fu,
        0x64u, 0x69u, 0x6Eu, 0x67u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x05u, 0x55u, 0x54u, 0x46u,
        0x5Fu, 0x38u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x10u, 0x63u, 0x6Fu, 0x64u, 0x65u, 0x63u,
        0x46u, 0x6Fu, 0x72u, 0x44u, 0x65u, 0x63u, 0x6Fu, 0x64u, 0x69u, 0x6Eu, 0x67u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u,
        0x00u, 0x00u, 0x00u, 0x05u, 0x55u, 0x54u, 0x46u, 0x5Fu, 0x38u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u,
        0x00u, 0x0Au, 0x69u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x74u, 0x79u, 0x49u, 0x64u, 0x00u, 0x00u, 0x00u, 0x7Fu,
        0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x74u, 0x79u, 0x49u, 0x64u, 0x00u,
        0xFFu, 0xFFu, 0xFFu, 0xFFu, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x69u, 0x73u, 0x43u,
        0x6Fu, 0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x65u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Fu, 0x63u, 0x6Fu, 0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x69u, 0x6Fu,
        0x6Eu, 0x53u, 0x74u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x02u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Fu, 0x75u, 0x73u, 0x65u, 0x52u, 0x61u, 0x6Eu, 0x64u, 0x6Fu,
        0x6Du, 0x53u, 0x65u, 0x72u, 0x76u, 0x65u, 0x72u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x70u, 0x65u, 0x72u, 0x66u, 0x6Fu, 0x72u, 0x6Du, 0x00u, 0x00u, 0x00u,
        0x0Bu, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Fu, 0x75u,
        0x73u, 0x65u, 0x41u, 0x75u, 0x74u, 0x6Fu, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x66u, 0x79u, 0x00u, 0x00u,
        0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x13u, 0x61u, 0x75u, 0x74u,
        0x6Fu, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x66u, 0x79u, 0x53u, 0x65u, 0x72u, 0x76u, 0x69u, 0x63u, 0x65u,
        0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u,
        0x00u, 0x14u, 0x61u, 0x75u, 0x74u, 0x6Fu, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x66u, 0x79u, 0x50u, 0x61u,
        0x73u, 0x73u, 0x77u, 0x6Fu, 0x72u, 0x64u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x75u, 0x73u, 0x65u, 0x53u, 0x61u, 0x73u, 0x6Cu, 0x00u,
        0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x73u, 0x61u,
        0x73u, 0x6Cu, 0x41u, 0x63u, 0x63u, 0x6Fu, 0x75u, 0x6Eu, 0x74u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x73u, 0x61u, 0x73u, 0x6Cu, 0x50u,
        0x61u, 0x73u, 0x73u, 0x77u, 0x6Fu, 0x72u, 0x64u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x10u, 0x75u, 0x73u, 0x65u, 0x41u, 0x75u, 0x74u, 0x6Fu,
        0x52u, 0x65u, 0x63u, 0x6Fu, 0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x15u, 0x61u, 0x75u, 0x74u, 0x6Fu, 0x52u, 0x65u, 0x63u, 0x6Fu,
        0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x49u, 0x6Eu, 0x74u, 0x65u, 0x72u, 0x76u, 0x61u, 0x6Cu, 0x00u, 0x00u, 0x00u,
        0x03u, 0x00u, 0x00u, 0x00u, 0x00u, 0x3Cu, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x61u,
        0x75u, 0x74u, 0x6Fu, 0x52u, 0x65u, 0x63u, 0x6Fu, 0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x52u, 0x65u, 0x74u, 0x72u,
        0x69u, 0x65u, 0x73u, 0x00u, 0x00u, 0x00u, 0x85u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u,
        0x00u, 0x00u, 0x19u, 0x75u, 0x6Eu, 0x6Cu, 0x69u, 0x6Du, 0x69u, 0x74u, 0x65u, 0x64u, 0x52u, 0x65u, 0x63u, 0x6Fu,
        0x6Eu, 0x6Eu, 0x65u, 0x63u, 0x74u, 0x52u, 0x65u, 0x74u, 0x72u, 0x69u, 0x65u, 0x73u, 0x00u, 0x00u, 0x00u, 0x01u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Eu, 0x72u, 0x65u, 0x6Au, 0x6Fu, 0x69u,
        0x6Eu, 0x43u, 0x68u, 0x61u, 0x6Eu, 0x6Eu, 0x65u, 0x6Cu, 0x73u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x75u, 0x73u, 0x65u, 0x43u, 0x75u, 0x73u, 0x74u, 0x6Fu,
        0x6Du, 0x4Du, 0x65u, 0x73u, 0x73u, 0x61u, 0x67u, 0x65u, 0x52u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x01u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x10u, 0x6Du, 0x73u, 0x67u, 0x52u, 0x61u,
        0x74u, 0x65u, 0x42u, 0x75u, 0x72u, 0x73u, 0x74u, 0x53u, 0x69u, 0x7Au, 0x65u, 0x00u, 0x00u, 0x00u, 0x03u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x05u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x13u, 0x6Du, 0x73u, 0x67u,
        0x52u, 0x61u, 0x74u, 0x65u, 0x4Du, 0x65u, 0x73u, 0x73u, 0x61u, 0x67u, 0x65u, 0x44u, 0x65u, 0x6Cu, 0x61u, 0x79u,
        0x00u, 0x00u, 0x00u, 0x03u, 0x00u, 0x00u, 0x00u, 0x08u, 0x98u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u,
        0x00u, 0x14u, 0x75u, 0x6Eu, 0x6Cu, 0x69u, 0x6Du, 0x69u, 0x74u, 0x65u, 0x64u, 0x4Du, 0x65u, 0x73u, 0x73u, 0x61u,
        0x67u, 0x65u, 0x52u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu,
        0x00u, 0x00u, 0x00u, 0x00u, 0x08u, 0x73u, 0x6Bu, 0x69u, 0x70u, 0x43u, 0x61u, 0x70u, 0x73u, 0x00u, 0x00u, 0x00u,
        0x0Bu, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x08u, 0x53u,
        0x75u, 0x70u, 0x70u, 0x6Fu, 0x72u, 0x74u, 0x73u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x53u, 0x65u, 0x72u, 0x76u, 0x65u, 0x72u, 0x4Cu,
        0x69u, 0x73u, 0x74u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu,
        0x00u, 0x00u, 0x00u, 0x00u, 0x04u, 0x43u, 0x61u, 0x70u, 0x73u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x43u, 0x61u, 0x70u, 0x73u, 0x45u,
        0x6Eu, 0x61u, 0x62u, 0x6Cu, 0x65u, 0x64u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x13u, 0x49u, 0x72u, 0x63u, 0x55u, 0x73u, 0x65u, 0x72u, 0x73u,
        0x41u, 0x6Eu, 0x64u, 0x43u, 0x68u, 0x61u, 0x6Eu, 0x6Eu, 0x65u, 0x6Cu, 0x73u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x02u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x55u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x72u,
        0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x10u, 0x00u,
        0x43u, 0x00u, 0x68u, 0x00u, 0x61u, 0x00u, 0x6Eu, 0x00u, 0x6Eu, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x73u, 0x00u,
        0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
      )
    )
  }
}
